@@ -33,7 +33,7 @@ adjust_left_type <- function(x, value)
     S4Vectors:::recycleVector(value, index_len)
 }
 
-.subassign_SVT_by_Lindex <- function(x, Lindex, value, old=FALSE)
+.subassign_SVT_by_Lindex <- function(x, Lindex, value)
 {
     x <- adjust_left_type(x, value)
     stopifnot(is.vector(Lindex), is.numeric(Lindex))
@@ -43,19 +43,13 @@ adjust_left_type <- function(x, value)
         return(x)
 
     value <- .normalize_right_value(value, type(x), length(Lindex))
-
-    if (old) {
-        new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex_OLD",
-                                    x@dim, x@type, x@SVT, Lindex, value)
-    } else {
-        new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex",
-                                    x@dim, x@type, x@SVT, FALSE, Lindex, value)
-    }
+    new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex",
+                                x@dim, x@type, x@SVT, FALSE, Lindex, value)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
 setMethod("subassign_Array_by_Lindex", "SVT_SparseArray",
-    function(x, Lindex, value) .subassign_SVT_by_Lindex(x, Lindex, value)
+    .subassign_SVT_by_Lindex
 )
 
 .subassign_SVT_by_Mindex <- function(x, Mindex, value)
@@ -68,11 +62,8 @@ setMethod("subassign_Array_by_Lindex", "SVT_SparseArray",
         return(x)
 
     value <- .normalize_right_value(value, type(x), nrow(Mindex))
-
-    if (storage.mode(Mindex) != "integer")
-        storage.mode(Mindex) <- "integer"
-    new_SVT <- SparseArray.Call("C_subassign_SVT_by_Mindex_OLD",
-                                x@dim, x@type, x@SVT, Mindex, value)
+    new_SVT <- SparseArray.Call("C_subassign_SVT_by_Mindex",
+                                x@dim, x@type, x@SVT, FALSE, Mindex, value)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
@@ -141,6 +132,20 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
+.adjust_right_array_dim <- function(right_array, selection_dim)
+{
+    right_dim <- unname(dim(right_array))
+    if (identical(selection_dim, right_dim))
+        return(right_array)
+    effdim_idx1 <- which(selection_dim != 1L)
+    effdim_idx2 <- which(right_dim != 1L)
+    if (!identical(selection_dim[effdim_idx1], right_dim[effdim_idx2]))
+        stop(wmsg("dimensions of right array don't ",
+                  "match dimensions of array selection"))
+    dim(right_array) <- selection_dim
+    right_array
+}
+
 .Nindex2Noffs <- function(Nindex)
 {
     stopifnot(is.list(Nindex))
@@ -162,9 +167,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    if (!identical(selection_dim, unname(dim(Rarray))))
-        stop(wmsg("dimensions of right array don't ",
-                  "match dimensions of array selection"))
+    Rarray <- .adjust_right_array_dim(Rarray, selection_dim)
     if (any(selection_dim == 0L))
         return(x)
 
@@ -180,35 +183,33 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-.subassign_SVT_with_SVT <- function(x, Nindex, v)
+.subassign_SVT_with_SVT <- function(x, Nindex, y)
 {
     stopifnot(is(x, "SVT_SparseArray"), is.list(Nindex))
     check_svt_version(x)
-    stopifnot(is(v, "SVT_SparseArray"))
-    check_svt_version(v)
+    stopifnot(is(y, "SVT_SparseArray"))
+    check_svt_version(y)
 
     ## Change 'x' type if necessary.
-    new_type <- type(c(vector(type(x)), vector(type(v))))
+    new_type <- type(c(vector(type(x)), vector(type(y))))
     type(x) <- new_type
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    if (!identical(selection_dim, unname(dim(v))))
-        stop(wmsg("dimensions of right array don't ",
-                  "match dimensions of array selection"))
+    y <- .adjust_right_array_dim(y, selection_dim)
     if (any(selection_dim == 0L))
         return(x)
 
-    ## Prepare 'Noffs' and 'v'.
+    ## Prepare 'Noffs' and 'y'.
     Norder <- S4Arrays:::get_Nindex_order(Nindex)
     Nindex <- S4Arrays:::subset_Nindex_by_Nindex(Nindex, Norder)
     Noffs <- .Nindex2Noffs(Nindex)
-    v <- S4Arrays:::subset_by_Nindex(v, Norder)
-    type(v) <- new_type
+    y <- S4Arrays:::subset_by_Nindex(y, Norder)
+    type(y) <- new_type
 
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_SVT",
-                                x@dim, x@type, x@SVT, Noffs,
-                                v@dim, v@type, v@SVT)
+                                x@dim, x@type, x@SVT, FALSE, Noffs,
+                                y@dim, y@type, y@SVT, FALSE)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
