@@ -5,16 +5,15 @@
 
 
 /* The only reason that we define 'RbyteNA' and 'listNA' is to make
-   'DEFINE_subassign_typeSV_with_SV_FUN(Rbyte)' and
-   'DEFINE_subassign_typeSV_with_SV_FUN(list)' work. Note that the
-   DEFINE_subassign_typeSV_with_SV_FUN() macro contains the following line:
+   'DEFINE_subassign_typeSV_FUN(Rbyte)' and other macros defined in
+   this file work. Note that these macros contain the following line:
 
-       out_type bg_val = out_sv->bg_is_na ? type ## NA : type ## 0;
+       type bg_val = out_sv->na_background ? type ## NA : type ## 0;
 
    So it absolutely doesn't matter what value we set 'RbyteNA' and 'listNA'
    to because we don't support NaArray objects of type raw or list.
    This means that if SparseVec 'out_sv' is of type raw or list
-   then 'out_sv->bg_is_na' is guaranteed to be FALSE.
+   then 'out_sv->na_background' is guaranteed to be FALSE.
    In other words, 'RbyteNA' and 'listNA' will **never** be used! */
 
 #define RbyteNA Rbyte0  /* exact value doesn't matter, see above */
@@ -44,7 +43,7 @@ static SEXPTYPE get_SV_subassign_Rtype(SEXPTYPE expected_Rtype,
 #define DEFINE_next_subassign_typeSV_out_val_FUN(type)			   \
 static inline int next_subassign_ ## type ## SV_out_val(		   \
 		const SparseVec *sv1, const int *offs, int n,		   \
-		const type *vals2, int cycle_len, const int *selection2,   \
+		const type *vals2, const int *selection2,		   \
 		int *k1, int *k, int *out_off, type *out_val)		   \
 {									   \
 	int ret = next_offset(sv1->nzoffs, get_SV_nzcount(sv1),		   \
@@ -56,17 +55,11 @@ static inline int next_subassign_ ## type ## SV_out_val(		   \
 		break;							   \
 	    case 3:							   \
 		(*k1)++;						   \
-	    case 2: {							   \
-		int i2;							   \
-		if (selection2 == NULL) {				   \
-			i2 = cycle_len ? *k % cycle_len : *k;		   \
-		} else {						   \
-			i2 = selection2[*k];				   \
-		}							   \
-		*out_val = vals2[i2];					   \
+	    case 2:							   \
+		int i = selection2 == NULL ? *k : selection2[*k];	   \
+		*out_val = vals2[i];					   \
 		(*k)++;							   \
 		break;							   \
-	    }								   \
 	}								   \
 	return ret;							   \
 }
@@ -78,8 +71,7 @@ DEFINE_next_subassign_typeSV_out_val_FUN(Rbyte)
 
 static inline int next_subassign_characterSV_out_val(
 		const SparseVec *sv1, const int *offs, int n,
-		SEXP Rvector, R_xlen_t block_offset, int cycle_len,
-		const int *selection,
+		SEXP Rvector, R_xlen_t block_offset, const int *selection,
 		int *k1, int *k, int *out_off, SEXP *out_val)
 {
 	int ret = next_offset(sv1->nzoffs, get_SV_nzcount(sv1),
@@ -92,12 +84,7 @@ static inline int next_subassign_characterSV_out_val(
 	    case 3:
 		(*k1)++;
 	    case 2: {
-		int i;
-		if (selection == NULL) {
-			i = block_offset + (cycle_len ? *k % cycle_len : *k);
-		} else {
-			i = selection[*k];
-		}
+		int i = selection == NULL ? block_offset + *k : selection[*k];
 		*out_val = STRING_ELT(Rvector, i);
 		(*k)++;
 		break;
@@ -108,8 +95,7 @@ static inline int next_subassign_characterSV_out_val(
 
 static inline int next_subassign_listSV_out_val(
 		const SparseVec *sv1, const int *offs, int n,
-		SEXP Rvector, R_xlen_t block_offset, int cycle_len,
-		const int *selection,
+		SEXP Rvector, R_xlen_t block_offset, const int *selection,
 		int *k1, int *k, int *out_off, SEXP *out_val)
 {
 	int ret = next_offset(sv1->nzoffs, get_SV_nzcount(sv1),
@@ -122,12 +108,7 @@ static inline int next_subassign_listSV_out_val(
 	    case 3:
 		(*k1)++;
 	    case 2: {
-		int i;
-		if (selection == NULL) {
-			i = block_offset + (cycle_len ? *k % cycle_len : *k);
-		} else {
-			i = selection[*k];
-		}
+		int i = selection == NULL ? block_offset + *k : selection[*k];
 		*out_val = VECTOR_ELT(Rvector, i);
 		(*k)++;
 		break;
@@ -141,35 +122,18 @@ static inline int next_subassign_listSV_out_val(
  * Inline functions process_subassign_<type>SV_out_val()
  */
 
-static inline int int_equal(int x, int y)
-{
-	return x == y;
-}
-static inline int double_equal(double x, double y)
-{
-	return x == y;
-}
-static inline int Rcomplex_equal(Rcomplex x, Rcomplex y)
-{
-	return x.r == y.r && x.i == y.i;
-}
-static inline int Rbyte_equal(Rbyte x, Rbyte y)
-{
-	return x == y;
-}
-
 #define DEFINE_process_subassign_typeSV_out_val_FUN(type)		   \
 static inline int process_subassign_ ## type ## SV_out_val(int ret,	   \
 		type out_val, int out_off, SparseVec *out_sv,		   \
-		const SparseVec *sv1, int k1)				   \
+		type bg_val, const SparseVec *sv1, int k1)		   \
 {									   \
 	int is_effrep = 0;						   \
 	if (ret == 2) {							   \
-		if (is_ ## type ## _bg(out_val, out_sv->bg_is_na))	   \
+		if (type ## _equal(out_val, bg_val))			   \
 			return 0;  /* zero replaces zero */		   \
 		is_effrep = 1;     /* nonzero replaces zero */		   \
 	} else if (ret == 3) {						   \
-		if (is_ ## type ## _bg(out_val, out_sv->bg_is_na))	   \
+		if (type ## _equal(out_val, bg_val))			   \
 			return 1;  /* zero replaces nonzero */		   \
 		/* nonzero replaces nonzero */				   \
 		type v1 = get_ ## type ## SV_nzval(sv1, k1 - 1);	   \
@@ -201,14 +165,14 @@ DEFINE_process_subassign_typeSV_out_val_FUN(Rbyte)
    leaf is not modified by the subassignment. */
 static inline int process_subassign_characterSV_out_val(int ret,
 		SEXP out_val, int out_off, SparseVec *out_sv,
-		const SparseVec *sv1, int k1)
+		SEXP bg_val, const SparseVec *sv1, int k1)
 {
 	int is_effrep = 1;
 	if (ret == 2) {
-		if (is_character_bg(out_val, out_sv->bg_is_na))
+		if (out_val == bg_val)
 			return 0;
 	} else if (ret == 3) {
-		if (is_character_bg(out_val, out_sv->bg_is_na))
+		if (out_val == bg_val)
 			return 1;
 		SEXP v1 = get_characterSV_nzval(sv1, k1 - 1);
 		/* See note above about this comparison. */
@@ -241,14 +205,14 @@ static inline int process_subassign_characterSV_out_val(int ret,
    as 'svt1@SVT'). */
 static inline int process_subassign_listSV_out_val(int ret,
 		SEXP out_val, int out_off, SparseVec *out_sv,
-		const SparseVec *sv1, int k1)
+		SEXP bg_val, const SparseVec *sv1, int k1)
 {
 	int is_effrep = 1;
 	if (ret == 2) {
-		if (is_list_bg(out_val, out_sv->bg_is_na))
+		if (out_val == bg_val)
 			return 0;
 	} else if (ret == 3) {
-		if (is_list_bg(out_val, out_sv->bg_is_na))
+		if (out_val == bg_val)
 			return 1;
 		SEXP v1 = get_listSV_nzval(sv1, k1 - 1);
 		/* See note above about this comparison. */
@@ -269,19 +233,20 @@ static inline int process_subassign_listSV_out_val(int ret,
 #define DEFINE_subassign_typeSV_FUN(type)				   \
 static int subassign_ ## type ## SV(					   \
 		const SparseVec *sv1, const int *offs, int n,		   \
-		const type *vals2, int cycle_len, const int *selection2,   \
+		const type *vals2, const int *selection2,		   \
 		SparseVec *out_sv)					   \
 {									   \
+	type bg_val = out_sv->na_background ? type ## NA : type ## 0;	   \
 	out_sv->nzcount = 0;						   \
 	int neffrep = 0, ret, k1 = 0, k = 0, out_off;			   \
 	type out_val;							   \
 	while ((ret = next_subassign_ ## type ## SV_out_val(sv1, offs, n,  \
-				vals2, cycle_len, selection2,		   \
-				&k1, &k, &out_off, &out_val)))		   \
+					vals2, selection2,		   \
+					&k1, &k, &out_off, &out_val)))	   \
 	{								   \
 		neffrep += process_subassign_ ## type ## SV_out_val(ret,   \
 				out_val, out_off, out_sv,		   \
-				sv1, k1);				   \
+				bg_val, sv1, k1);			   \
 	}								   \
 	return neffrep;							   \
 }
@@ -293,40 +258,39 @@ DEFINE_subassign_typeSV_FUN(Rbyte)
 
 static int subassign_characterSV(
 		const SparseVec *sv1, const int *offs, int n,
-		SEXP Rvector, R_xlen_t block_offset, int cycle_len,
-		const int *selection,
+		SEXP Rvector, R_xlen_t block_offset, const int *selection,
 		SparseVec *out_sv)
 {
+	SEXP bg_val = out_sv->na_background ? characterNA : character0;
 	out_sv->nzcount = 0;
 	int neffrep = 0, ret, k1 = 0, k = 0, out_off;
 	SEXP out_val;
 	while ((ret = next_subassign_characterSV_out_val(sv1, offs, n,
-				Rvector, block_offset, cycle_len, selection,
-				&k1, &k, &out_off, &out_val)))
+					Rvector, block_offset, selection,
+					&k1, &k, &out_off, &out_val)))
 	{
 		neffrep += process_subassign_characterSV_out_val(ret,
 				out_val, out_off, out_sv,
-				sv1, k1);
+				bg_val, sv1, k1);
 	}
 	return neffrep;
 }
 
 static int subassign_listSV(
 		const SparseVec *sv1, const int *offs, int n,
-		SEXP Rvector, R_xlen_t block_offset, int cycle_len,
-		const int *selection,
+		SEXP Rvector, R_xlen_t block_offset, const int *selection,
 		SparseVec *out_sv)
 {
 	out_sv->nzcount = 0;
 	int neffrep = 0, ret, k1 = 0, k = 0, out_off;
 	SEXP out_val;
 	while ((ret = next_subassign_listSV_out_val(sv1, offs, n,
-				Rvector, block_offset, cycle_len, selection,
-				&k1, &k, &out_off, &out_val)))
+					Rvector, block_offset, selection,
+					&k1, &k, &out_off, &out_val)))
 	{
 		neffrep += process_subassign_listSV_out_val(ret,
 				out_val, out_off, out_sv,
-				sv1, k1);
+				list0, sv1, k1);
 	}
 	return neffrep;
 }
@@ -336,49 +300,35 @@ static int subassign_SV(
 		SEXP Rvector, R_xlen_t block_offset, const int *selection,
 		SparseVec *out_sv)
 {
-	if (Rvector == R_NilValue)
+	if (selection != NULL && block_offset != 0)
 		error("SparseArray internal error in subassign_SV():\n"
-		      "    'Rvector' cannot be NULL");
+		      "    'block_offset' must be 0 when 'selection != NULL'");
 	SEXPTYPE Rtype = get_SV_subassign_Rtype(TYPEOF(Rvector), sv1, out_sv);
-	int Rvector_len = LENGTH(Rvector), cycle_len = 0;
-	if (block_offset != 0) {
-		if (selection != NULL)
-			error("SparseArray internal error in subassign_SV():\n"
-			      "    block_offset != 0 && selection != NULL");
-		if (Rvector_len < block_offset + n)
-			error("SparseArray internal error in subassign_SV():\n"
-			      "    LENGTH(Rvector) < block_offset + n");
-	} else if (selection == NULL && Rvector_len < n) {
-		if (Rvector_len == 0)
-			error("SparseArray internal error in subassign_SV():\n"
-			      "    LENGTH(Rvector) == 0");
-		cycle_len = Rvector_len;
-	}
 	switch (Rtype) {
 	    case INTSXP: case LGLSXP:
 		return subassign_intSV(sv1, offs, n,
-				INTEGER(Rvector) + block_offset, cycle_len,
-				selection, out_sv);
+				INTEGER(Rvector) + block_offset, selection,
+				out_sv);
 	    case REALSXP:
 		return subassign_doubleSV(sv1, offs, n,
-				REAL(Rvector) + block_offset, cycle_len,
-				selection, out_sv);
+				REAL(Rvector) + block_offset, selection,
+				out_sv);
 	    case CPLXSXP:
 		return subassign_RcomplexSV(sv1, offs, n,
-				COMPLEX(Rvector) + block_offset, cycle_len,
-				selection, out_sv);
+				COMPLEX(Rvector) + block_offset, selection,
+				out_sv);
 	    case RAWSXP:
 		return subassign_RbyteSV(sv1, offs, n,
-				RAW(Rvector) + block_offset, cycle_len,
-				selection, out_sv);
+				RAW(Rvector) + block_offset, selection,
+				out_sv);
 	    case STRSXP:
 		return subassign_characterSV(sv1, offs, n,
-				Rvector, block_offset, cycle_len,
-				selection, out_sv);
+				Rvector, block_offset, selection,
+				out_sv);
 	    case VECSXP:
 		return subassign_listSV(sv1, offs, n,
-				Rvector, block_offset, cycle_len,
-				selection, out_sv);
+				Rvector, block_offset, selection,
+				out_sv);
 	}
 	error("SparseArray internal error in subassign_SV():\n"
 	      "    'out_sv' of type \"%s\" not supported", type2char(Rtype));
@@ -397,10 +347,6 @@ static int subassign_SV(
    must be < 'sv->len'.
    The replacement value (a.k.a. right value) is the block of 'n' elements
    in 'Rvector' that starts at offset 'block_offset'. It can contain zeros.
-   About the length of 'Rvector':
-   - If 'block_offset != 0' then its length must be >= 'block_offset + n'.
-   - If 'block_offset == 0' then 'Rvector' can be of any length (except 0
-     unless 'n' is also 0) and will be recycled if its length is < 'n'.
    Returns the number of **effective** replacements, that is, the number of
    offsets for which the subassignment operation effectively modifies the
    original value. */
@@ -474,8 +420,8 @@ static int subassign_ ## type ## SV_with_SV(				   \
 		const SparseVec *sv1, const int *offs,			   \
 		const SparseVec *sv2, SparseVec *out_sv)		   \
 {									   \
+	out_type bg_val = out_sv->na_background ? type ## NA : type ## 0;  \
 	out_sv->nzcount = 0;						   \
-	out_type bg_val = out_sv->bg_is_na ? type ## NA : type ## 0;	   \
 	int neffrep = 0, ret, k1 = 0, k = 0, k2 = 0, out_off;		   \
 	out_type out_val;						   \
 	while ((ret = next_subassign_ ## type ## SV_with_SV_out_val(	   \
@@ -484,7 +430,7 @@ static int subassign_ ## type ## SV_with_SV(				   \
 	{								   \
 		neffrep += process_subassign_ ## type ## SV_out_val(ret,   \
 				out_val, out_off, out_sv,		   \
-				sv1, k1);				   \
+				bg_val, sv1, k1);			   \
 	}								   \
 	return neffrep;							   \
 }
@@ -533,9 +479,9 @@ int _subassign_SV_with_SV(
  * subassign_full_<type>SV_with_Rvector_block()
  */
 
-static inline int next_k1(const SparseVec *sv1, int *k1, int out_off)
+static inline int next_k1(const SparseVec *sv1, int *k1, int off)
 {
-	if (*k1 < get_SV_nzcount(sv1) && sv1->nzoffs[*k1] == out_off) {
+	if (*k1 < get_SV_nzcount(sv1) && sv1->nzoffs[*k1] == off) {
 		(*k1)++;
 		return 3;
 	}
@@ -545,17 +491,17 @@ static inline int next_k1(const SparseVec *sv1, int *k1, int out_off)
 #define DEFINE_subassign_full_typeSV_with_Rvector_block_FUN(type)	  \
 static int subassign_full_ ## type ## SV_with_Rvector_block(		  \
 		const SparseVec *sv1,					  \
-		const type *vals2, int cycle_len, SparseVec *out_sv)	  \
+		const type *vals2, SparseVec *out_sv)			  \
 {									  \
+	type bg_val = out_sv->na_background ? type ## NA : type ## 0;	  \
 	out_sv->nzcount = 0;						  \
 	int neffrep = 0, k1 = 0;					  \
-	for (int out_off = 0; out_off < out_sv->len; out_off++) {	  \
-		int ret = next_k1(sv1, &k1, out_off);			  \
-		int i = cycle_len ? out_off % cycle_len : out_off;	  \
+	for (int i = 0; i < out_sv->len; i++) {				  \
+		int ret = next_k1(sv1, &k1, i);				  \
 		type out_val = vals2[i];				  \
 		neffrep += process_subassign_ ## type ## SV_out_val(ret,  \
-				out_val, out_off, out_sv,		  \
-				sv1, k1);				  \
+				out_val, i, out_sv,			  \
+				bg_val, sv1, k1);			  \
 	}								  \
 	return neffrep;							  \
 }
@@ -566,37 +512,32 @@ DEFINE_subassign_full_typeSV_with_Rvector_block_FUN(Rcomplex)
 DEFINE_subassign_full_typeSV_with_Rvector_block_FUN(Rbyte)
 
 static int subassign_full_characterSV_with_Rvector_block(const SparseVec *sv1,
-		SEXP Rvector, R_xlen_t block_offset, int cycle_len,
-		SparseVec *out_sv)
+		SEXP Rvector, R_xlen_t block_offset, SparseVec *out_sv)
 {
+	SEXP bg_val = out_sv->na_background ? characterNA : character0;
 	out_sv->nzcount = 0;
 	int neffrep = 0, k1 = 0;
-	for (int out_off = 0; out_off < out_sv->len; out_off++) {
-		int ret = next_k1(sv1, &k1, out_off);
-		int i = cycle_len ? out_off % cycle_len :
-				    block_offset + out_off;
-		SEXP out_val = STRING_ELT(Rvector, i);
+	for (int i = 0; i < out_sv->len; i++) {
+		int ret = next_k1(sv1, &k1, i);
+		SEXP out_val = STRING_ELT(Rvector, block_offset + i);
 		neffrep += process_subassign_characterSV_out_val(ret,
-				out_val, out_off, out_sv,
-				sv1, k1);
+				out_val, i, out_sv,
+				bg_val, sv1, k1);
 	}
 	return neffrep;
 }
 
 static int subassign_full_listSV_with_Rvector_block(const SparseVec *sv1,
-		SEXP Rvector, R_xlen_t block_offset, int cycle_len,
-		SparseVec *out_sv)
+		SEXP Rvector, R_xlen_t block_offset, SparseVec *out_sv)
 {
 	out_sv->nzcount = 0;
 	int neffrep = 0, k1 = 0;
-	for (int out_off = 0; out_off < out_sv->len; out_off++) {
-		int ret = next_k1(sv1, &k1, out_off);
-		int i = cycle_len ? out_off % cycle_len :
-				    block_offset + out_off;
-		SEXP out_val = VECTOR_ELT(Rvector, i);
+	for (int i = 0; i < out_sv->len; i++) {
+		int ret = next_k1(sv1, &k1, i);
+		SEXP out_val = VECTOR_ELT(Rvector, block_offset + i);
 		neffrep += process_subassign_listSV_out_val(ret,
-				out_val, out_off, out_sv,
-				sv1, k1);
+				out_val, i, out_sv,
+				list0, sv1, k1);
 	}
 	return neffrep;
 }
@@ -618,63 +559,35 @@ static int subassign_full_listSV_with_Rvector_block(const SparseVec *sv1,
    of **effective** replacements (which it returns).
 
    'sv->len' and 'out_sv->len' must be the same. 'sv' can be lacunar.
-   The block of 'sv->len' (or 'out_sv->len') elements in 'Rvector' that
-   starts at offset 'block_offset' forms the replacement value (a.k.a.
-   right value) of the subassignment operation. It can contain zeros.
-   About the length of 'Rvector':
-   - If 'block_offset != 0' then its length must be >= 'block_offset + sv->len'.
-   - If 'block_offset == 0' then 'Rvector' can be of any length (except 0
-     unless 'sv->len' is also 0) and will be recycled if its length
-     is < 'sv->len'.
+   The block of 'n' elements in 'Rvector' that starts at offset 'block_offset'
+   forms the replacement value (a.k.a. right value) of the subassignment
+   operation. It can contain zeros.
    Returns the number of **effective** replacements, that is, the number of
    offsets for which the subassignment operation effectively modifies the
    original value. */
 int _subassign_full_SV_with_Rvector_block(const SparseVec *sv,
 		SEXP Rvector, R_xlen_t block_offset, SparseVec *out_sv)
 {
-	if (Rvector == R_NilValue)
-		error("SparseArray internal error in "
-		      "_subassign_full_SV_with_Rvector_block():\n"
-		      "    'Rvector' cannot be NULL");
 	SEXPTYPE Rtype = get_SV_subassign_Rtype(TYPEOF(Rvector), sv, out_sv);
-	int Rvector_len = LENGTH(Rvector), cycle_len = 0;
-	if (block_offset != 0) {
-		if (Rvector_len < block_offset + sv->len)
-			error("SparseArray internal error in "
-			      "_subassign_full_SV_with_Rvector_block():\n"
-			      "    LENGTH(Rvector) < block_offset + sv->len");
-	} else if (Rvector_len < sv->len) {
-		if (Rvector_len == 0)
-			error("SparseArray internal error in "
-			      "_subassign_full_SV_with_Rvector_block():\n"
-			      "    LENGTH(Rvector) == 0");
-		cycle_len = Rvector_len;
-	}
 	switch (Rtype) {
 	    case INTSXP: case LGLSXP:
 		return subassign_full_intSV_with_Rvector_block(sv,
-				INTEGER(Rvector) + block_offset, cycle_len,
-				out_sv);
+				INTEGER(Rvector) + block_offset, out_sv);
 	    case REALSXP:
 		return subassign_full_doubleSV_with_Rvector_block(sv,
-				REAL(Rvector) + block_offset, cycle_len,
-				out_sv);
+				REAL(Rvector) + block_offset, out_sv);
 	    case CPLXSXP:
 		return subassign_full_RcomplexSV_with_Rvector_block(sv,
-				COMPLEX(Rvector) + block_offset, cycle_len,
-				out_sv);
+				COMPLEX(Rvector) + block_offset, out_sv);
 	    case RAWSXP:
 		return subassign_full_RbyteSV_with_Rvector_block(sv,
-				RAW(Rvector) + block_offset, cycle_len,
-				out_sv);
+				RAW(Rvector) + block_offset, out_sv);
 	    case STRSXP:
 		return subassign_full_characterSV_with_Rvector_block(sv,
-				Rvector, block_offset, cycle_len,
-				out_sv);
+				Rvector, block_offset, out_sv);
 	    case VECSXP:
 		return subassign_full_listSV_with_Rvector_block(sv,
-				Rvector, block_offset, cycle_len,
-				out_sv);
+				Rvector, block_offset, out_sv);
 	}
 	error("SparseArray internal error in "
 	      "_subassign_full_SV_with_Rvector_block():\n"
